--[[Author:		DieenDieen 
	License:	All Rights Reserved
	Contact:	
-- global lookup
]]

local folder, core = ...
MO = core --global table

-- local
title		= "Mail Outbox"
version		= GetAddOnMetadata(folder, "X-Curse-Packaged-Version") or ""
titleFull	= title.." "..version
packedtitle = "MailOutbox";

outgoingmail = {};
outgoingmailitems = {};
outgoingmailmoney = 0;
outgoingmailCOD = 0;
MailOutboxHistory = {};
MailOutboxHistoryAvaiable = false;

mooptions = {};

-- Isolate the environment
local _G = getfenv(0)
setmetatable(MO, {__index = _G})
setfenv(1, MO)


core = LibStub("AceAddon-3.0"):NewAddon(packedtitle, "AceConsole-3.0",  "AceHook-3.0", "AceEvent-3.0") 

local AceGUI = LibStub("AceGUI-3.0")
local MOScrollingTable = LibStub("ScrollingTable")

moconfig = {
    name = packedtitle,
    handler = core,
    type = 'group',
    args = {
    	COD = {
        	type = 'group',
        	name = 'COD settings',
        	args = {
        		Zero={
        			type = 'toggle',            
        			name = 'Zero COD gold visible',
        			desc = 'Enables showing of zero COD gold in outgoing mail report',
        			set = 'SetOption',
        			get = 'GetOption',
        		},
        		Graphics={
        			type = 'toggle',            
        			name = 'Use graphics',
        			desc = 'Uses graphics for showing outgoing COD gold',
        			set = 'SetOption',
        			get = 'GetOption',
        		},        		
            }
        },
        Gold = {
        	type = 'group',
        	name = 'Gold settings',
        	args = {
        		Zero={
        			type = 'toggle',            
        			name = 'Zero gold visible',
        			desc = 'Enables showing of zero gold in outgoing mail report',
        			set = 'SetOption',
        			get = 'GetOption',
        		},
        		Graphics={
        			type = 'toggle',            
        			name = 'Use graphics',
        			desc = 'Uses graphics for showing outgoing gold',
        			set = 'SetOption',
        			get = 'GetOption',
        		},        		
            },
        },
        History = {
        	type = 'group',
        	name = 'History setting',
        	args = {
        		Enabled={
        			type = 'toggle',            
        			name = 'Tracking enabled',
        			desc = 'Enables storing history of outgoing mails',
        			set = 'SetOption',
        			get = 'GetOption',
        		},
       		        		
            },
        },
    },
}


modefaultoptions = {
	version = 1,
	["COD"] = {
		["Zero"] = false,
		["Graphics"] = true,
	},
	["Gold"] = {
		["Zero"] = false,
		["Graphics"] = true,
	},
	["History"] = {
		["Enabled"] = true,
	},	
}




local regEvents = {
	"ADDON_LOADED",
	"MAIL_SEND_INFO_UPDATE",
	"SEND_MAIL_COD_CHANGED",
	"MAIL_SEND_SUCCESS",
	"SEND_MAIL_MONEY_CHANGED",
	"MAIL_CLOSED",
}

function core:OnInitialize()
    --print "---mailoutbox init";
    self:RegisterChatCommand("mailoutbox", "MySlashProcessorFunc");
    local config=LibStub("AceConfig-3.0");
  	local dialog = LibStub("AceConfigDialog-3.0");
	config:RegisterOptionsTable(packedtitle, moconfig);
	coreOpts = dialog:AddToBlizOptions(packedtitle, title);
	
end


function core:OnEnable()
   --print "---mailoutbox OnEnable";
   for i, event in pairs (regEvents) do 
		self:RegisterEvent(event)
	end	
end

function core:MAIL_CLOSED(event, ...)
	mailOpen = 0
end


function GetItemListString(aMail)
	local ItemList="";

	for index=1,#aMail.Items  do	              
		ItemList = ItemList.." "..(aMail.Items [index].Link).."x"..(aMail.Items [index].Count);
   	   end

	return ItemList;
end;


function core:ADDON_LOADED(event, ...)
    --print "--ADDON_LOADED event";
    if MailOutboxHistory==nil then
          MailOutboxHistory = {};
       end;
    MailOutboxHistoryAvaiable = true;
           
    if mooptions.version == nil then
       mooptions.version = modefaultoptions.version;
       mooptions.COD = modefaultoptions.COD;
       mooptions.Gold = modefaultoptions.Gold;
       --print "default options loaded";
       end;
    if mooptions.History == nil then mooptions.History = modefaultoptions.History; end;
    
end

function FormatMoneyTostring(ammount,category)
   local outstring="";
   if (ammount>0) or (mooptions[category].Zero) then
      if mooptions[category].Graphics then
         outstring=GetCoinTextureString (ammount);
      else
         outstring=tostring(ammount/10000).."g";
      end;
   end;
   return outstring; 
end;



function ShowHistory()
-- Create a container frame
local f = AceGUI:Create("Frame")
f:SetCallback("OnClose",function(widget)
local f=widget.ScrollTable.frame;
f:Hide();
widget.ScrollTable:SetData({});
f:UnregisterAllEvents();
f:ClearAllPoints();
widget.ScrollTable = nil;
AceGUI:Release(widget) 
end)
f:SetTitle("Mail Outbox history page");
f:SetStatusText("List of sent mails")
f:SetLayout("Fill")
--f.frame:SetResizable(false);

local mailhistorycols = {
	{ name= "Date/time", width = 160, defaultsort = "dsc", },
	{ name= "From", width = 100, defaultsort = "dsc",},
	{ name= "Recipient", width = 100, defaultsort = "dsc", },
	{ name= "Subject", width = 200, defaultsort = "dsc", },
	{ name= "Money", width = 100, defaultsort = "dsc",},
	{ name= "COD", width = 100, defaultsort = "dsc",},
	{ name= "#items", width = 40, defaultsort = "dsc",},
	{ name= "list", width = 200, defaultsort = "dsc",},
	};
	
  	
	local window  = f.frame
	
	local mailhistoryST = MOScrollingTable:CreateST(mailhistorycols, 10, 16, nil, window)
		mailhistoryST.frame:SetPoint("BOTTOMLEFT",window, 10,10)
		mailhistoryST.frame:SetPoint("TOP", window, 0, -60)
		mailhistoryST.frame:SetPoint("RIGHT", window, -10,0)	

	f.ScrollTable=mailhistoryST;
	mailhistoryST.Fire=function(...)return true;end;
	mailhistoryST.userdata={};
	
	
	if MailOutboxHistoryAvaiable then 
	local testdata={};
	local i=0;
	for index,sentmail in pairs(MailOutboxHistory) do
		if sentmail.Valid then
		  local Itemlist= GetItemListString(sentmail);		
		  tinsert(testdata, {cols = {
		  	{value = sentmail.Timestamp},
		  	{value = sentmail.From},
		  	{value = sentmail.Recipient},
		  	{value = sentmail.Subject},
		  	{value = FormatMoneyTostring(sentmail.Money,"Gold")},
		  	{value = FormatMoneyTostring(sentmail.COD,"COD")},
		  	{value = #sentmail.Items},{value = Itemlist}}});
		  end
     end;

	   mailhistoryST:SetData(testdata);		
	end
	
    local width = 100
	for i, data in pairs(mailhistorycols) do 
		width = width + data.width
	end
	f:SetWidth(width);
    mailhistoryST:SetDisplayRows((f.content.height / 16)-2, 16);
	
	--mailhistoryST:Show()


end;



function core:GetOption(info)
  local opt = mooptions[info[#info]];
  local optname=info[#info];
  if #info > 1 then
     if mooptions[info[#info-1]] == nil then
     	mooptions[info[#info-1]]={};
        end;
       opt = mooptions[info[#info-1]] [info[#info]];
       optname=info[#info-1].."."..info[#info];       
     end;
     
     --print("The " .. tostring(optname) .. " returned as: " .. tostring(opt) ); 
    return opt;
end

function core:SetOption(info, value)
  if #info > 1 then
 	 if  mooptions [info[#info-1]] == nil then
 	    mooptions [info[#info-1]]={};
 	    end;
     mooptions [info[#info-1]] [info[#info]] = value;
     --print("The " .. info[#info-1].."."..info[#info] .. " was set to: " .. tostring(value) );     
     else
     mooptions [info[#info]] = value;
     --print("The " .. info[#info] .. " was set to: " .. tostring(value) );     
     end;
  
end

function ResetSendMailInfo()
	outgoingmail = {};
	outgoingmail.Valid = false;		
	outgoingmailitems = {};
	outgoingmailCOD = 0;
	outgoingmailmoney = 0;	
end;


function reportMailInfo()
   if outgoingmail.Valid then
		outgoingmail.Timestamp = date("%Y/%m/%d %H:%M:%S");
		local countableitems = (#outgoingmail.Items).." item";
		if (#outgoingmail.Items > 1) then
		     countableitems = countableitems.."s";
		  elseif (#outgoingmail.Items == 0) then
		     countableitems = "no items";
		  end;
		--GetCoinTextureString(10001));
		local outmoney="";
		if (outgoingmail.Money>0) or (mooptions.Gold.Zero) then
		   if mooptions.Gold.Graphics then
			   outmoney=GetCoinTextureString (outgoingmail.Money);
			   else
			   outmoney=tostring(outgoingmail.Money/10000).."g";
			   end;
			end;
			
		local outCOD="";
		if (outgoingmail.COD>0) or (mooptions.COD.Zero) then
		   if mooptions.COD.Graphics then
			   outCOD="COD"..GetCoinTextureString (outgoingmail.COD);
			   else
			   outCOD="COD"..tostring(outgoingmail.COD/10000).."g";
			   end;
			end;
			
		ChatFrame1:AddMessage (outgoingmail.Timestamp..":(to:"..outgoingmail.Recipient..", "..countableitems..", "..outmoney.." "..outCOD..") '"..outgoingmail.Subject.."'");

		local ItemList=GetItemListString(outgoingmail);
		
		if (0<#outgoingmail.Items) then 
		   ChatFrame1:AddMessage (ItemList);
		   end;
		outgoingmail.Version = 1;   
		if mooptions.History.Enabled then
		   MailOutboxHistory [#MailOutboxHistory+1] = outgoingmail;
		   end;
	  ResetSendMailInfo();		   
   end;
end;


function UpdateSendMailInfo()
--[[
	print "------"; 
    print (SendMailSubjectEditBox:GetText());
	print (SendMailNameEditBox:GetText());
	print (SendMailBodyEditBox:GetText());
	print (SendMailMoneyText:GetText());
	print (MoneyInputFrame_GetCopper(SendMailMoney));
	]]
	outgoingmail = {};
	outgoingmail.From = GetUnitName ("player");
	outgoingmail.Recipient = SendMailNameEditBox:GetText();
	outgoingmail.Subject = SendMailSubjectEditBox:GetText();
	outgoingmail.Body = SendMailBodyEditBox:GetText();
	outgoingmail.Cost = GetSendMailPrice();
	outgoingmail.Items = outgoingmailitems;
	outgoingmail.COD = outgoingmailCOD;    
	outgoingmail.Money = outgoingmailmoney;
	outgoingmail.Valid = true;
end;

function UpdateSendMailitemsInfo()
   outgoingmailitems = {};
   for index=1, 12 do 
		Name, Texture, Count, Quality = GetSendMailItem (index); 
		if Name then    
			local ItemAlreadyInList = false;	
		    for inindex=1,#outgoingmailitems do
		       if outgoingmailitems[inindex].Name == Name then
		             outgoingmailitems[inindex].Count = outgoingmailitems[inindex].Count + Count;
		             ItemAlreadyInList = true;
		          end;
		    end;
		    if not ItemAlreadyInList then
				local NewItem= {};
				NewItem.Name=Name;
				NewItem.Count=Count;
				NewItem.Link=GetSendMailItemLink (index);
				outgoingmailitems [1+#outgoingmailitems] =NewItem;
				ItemAlreadyInList = true;
			end;
		end
	end
	UpdateSendMailInfo();
end;


function core:MAIL_SEND_SUCCESS(event, ...)
    -- print "------mail send succes";
    outgoingmailmoney = 0;
    outgoingmailCOD = 0;    
    reportMailInfo();

    -- print (outgoingmail.Money);
    -- print (outgoingmail.COD);    
    
    
    end


function core:SEND_MAIL_MONEY_CHANGED(event, ...)
   -- print "------SEND_MAIL_MONEY_CHANGED";
   
   if GetSendMailMoney()~=0 then
      outgoingmailmoney = GetSendMailMoney();
      end;
   UpdateSendMailInfo();
   
  -- print (outgoingmail.Money);
  -- print (outgoingmail.COD);
      
end

function core:SEND_MAIL_COD_CHANGED(event, ...)
  -- print "------SEND_MAIL_COD_CHANGED";   
    
  if GetSendMailCOD()~=0 then
      outgoingmailCOD = GetSendMailCOD();
   end;
      
    UpdateSendMailInfo();
    
    -- print (outgoingmail.Money);
    -- print (outgoingmail.COD);
end

function core:MAIL_SEND_INFO_UPDATE(event, ...)
   -- print "------MAIL_SEND_INFO_UPDATE";
   UpdateSendMailitemsInfo();	
end


function core:OnDisable()
	
end




----------------------------------------------
function core:MySlashProcessorFunc(input)	--
----------------------------------------------
    if input=="" then
      print ("--- Mail Outbox is running");
      print ("to check mail history use /mailoutbox history");
   end;
   if input=="history" then
     ShowHistory();
   end;
end





-----------------------------------------------------







